program Joystick;

{ $BOOTRST $0F000}         {Reset Jump to $0F000}
{$NOSHADOW}
{ $W+ Warnings}            {Warnings off}

Device = mega128, VCC=5;

Define_Fuses
  COMport   = USB;
  LockBits0 = [];
  FuseBits0 = [];
  FuseBits1 = [CKOPT, SPIEN, JTAGEN, OCDEN];
  FuseBits2 = [];
  ProgMode  = JTAG;

Import SysTick, ADCPort, LCDmultiPort, TWImaster, SwitchPort_G, BeepPort;

From System Import ;

Define
        ProcClock    = 16000000;        {Hertz}
        SysTick      = 10;              {msec}
        StackSize    = $0064, iData;
        FrameSize    = $0064, iData;
        
        ADCchans     = 2, iData;
        ADCpresc     = 128;
        
        BeepPort     = PortG, 4;
        
        TWIpresc     = TWI_BR400;
        LCDmultiPort = I2C_TWI;
        LCDrows_M    = 4;              {rows}
        LCDcolumns_M = 20;             {columns per line}
        LCDtype_M    = 44780;
        LCDBargraph1 = LCDmultiPort;
        LCDBargraph2 = LCDmultiPort;
        LCDBargraph3 = LCDmultiPort;
        LCDBargraph4 = LCDmultiPort;

        SwitchPort_G = [Center, PinF, 3][North, PinD, 7][East, PinD, 6]
                       [South, PinD, 5][West, PinD, 4];
        PolarityP_G  = 0;              // all low active
        Debounce     = 3;              // 3 Systicks

Implementation

{$IDATA}

{--------------------------------------------------------------}
{ Type Declarations }

type

{--------------------------------------------------------------}
{ Const Declarations }


{--------------------------------------------------------------}
{ Var Declarations }
{$IDATA}
var
  bb      : byte;

{--------------------------------------------------------------}
{ functions }
procedure ScanButtons;                      // digital Joystick
begin
  LCDbarSet1(2, 2, 8, 100);                 // x- display
  LCDbarSet2(2, 12, 8, 100);                // x+ display
  LCDbarSet3(3, 2, 8, 100);                 // y- display
  LCDbarSet4(3, 12, 8, 100);                // y+ display
  LCDxy_M(LCD_m1, 0, 1);
  Write(LCDOut_M, 'Digital Joystick');
  LCDxy_M(LCD_m1, 0, 2);
  Write(LCDOut_M, 'X-');
  LCDxy_M(LCD_m1, 10, 2);
  Write(LCDOut_M, 'X+');
  LCDxy_M(LCD_m1, 0, 3);
  Write(LCDOut_M, 'Y-');
  LCDxy_M(LCD_m1, 10, 3);
  Write(LCDOut_M, 'Y+');
  loop
    if INP_STABLE_G(Center) then
      BeepClick;
    endif;
    if INP_STABLE_G(West) then              // x-
      LCDbarOut1(100);
    else
      LCDbarOut1(0);
    endif;
    if INP_STABLE_G(East) then              // x+
      LCDbarOut2(100);
    else
      LCDbarOut2(0);
    endif;
    if INP_STABLE_G(North) then             // y+
      LCDbarOut4(100);
    else
      LCDbarOut4(0);
    endif;
    if INP_STABLE_G(South) then             // y-
      LCDbarOut3(100);
    else
      LCDbarOut3(0);
    endif;
    mDelay(100);
  endloop;
end;

procedure ScanPotis;                        // analog Joystick
begin
  LCDbarSet1(2, 2, 18, 100);                // x display
  LCDbarSet2(3, 2, 18, 100);                // y display
  LCDxy_M(LCD_m1, 0, 1);
  Write(LCDOut_M, 'Analog Joystick');
  LCDxy_M(LCD_m1, 0, 2);
  Write(LCDOut_M, 'X ');
  LCDxy_M(LCD_m1, 0, 3);
  Write(LCDOut_M, 'Y ');
  loop
    bb:= byte(GetADC(2) div 10);
    LCDbarOut1(bb);
    bb:= byte(GetADC(1) div 10);
    LCDbarOut2(bb);
    if INP_STABLE_G(Center) then
      BeepClick;
    endif;
    mDelay(100);
  endloop;
end;

{--------------------------------------------------------------}
{ Main Program }
{$IDATA}

begin
  PortF:= $ff;                              // pullups on
  PortD:= $ff;                              // pullups on
  EnableInts;

  LCDsetup_M(LCD_m1);

  LCDbarInit_M;                             // stores charset to display

  LCDcursor_M(LCD_m1, false, false);
  LCDclr_M(LCD_m1);
  Write(LCDOut_M, 'E-LAB Joystick Demo');
  LCDxy_M(LCD_m1, 0, 1);
  Write(LCDOut_M, 'Move any Axe');

  loop
    if (PORT_STABLE_G and $1F) <> 0 then
      ScanButtons;                           // digital Joystick
    elsif GetADC(1) < 600 then
      ScanPotis;                             // analog Joystick
    elsif GetADC(2) < 600 then
      ScanPotis;                             // analog Joystick
    endif;
  endloop;
end Joystick.

  LCDbarOut2(bb);


